package com.jwl.driver.server.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.StrUtil;
import com.jwl.driver.server.constant.Constants;
import com.jwl.driver.server.entity.OrderPayInfo;
import com.jwl.driver.server.entity.TdMember;
import com.jwl.driver.server.entity.TdSysUser;
import com.jwl.driver.server.enums.PayStatus;
import com.jwl.driver.server.exception.BusinessException;
import com.jwl.driver.server.mapper.OrderPayInfoMapper;
import com.jwl.driver.server.service.IOrderPayInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.service.ITdMemberService;
import com.jwl.driver.server.service.ITdSysUserService;
import com.jwl.driver.server.util.SecurityUtil;
import com.jwl.driver.server.vo.H5PayVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Objects;

/**
 * <p>
 * 支付表 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
@Slf4j
public class OrderPayInfoServiceImpl extends ServiceImpl<OrderPayInfoMapper, OrderPayInfo> implements IOrderPayInfoService {

    @Autowired
    private ITdSysUserService userService;

    @Autowired
    private ITdMemberService memberService;

    @Autowired
    private Snowflake snowflake;

    @Override
    public OrderPayInfo createOrderPayInfo(H5PayVo payVo) {
        Long userId = StrUtil.isNotBlank(payVo.getUserId()) ? Long.parseLong(payVo.getUserId()) : SecurityUtil.getUserId();
        TdSysUser user = userService.getById(userId);
        if (Objects.isNull(user)) {
            log.error("数据异常,用户不存在" + userId);
            throw new BusinessException("数据缺失");
        }
        TdMember member = memberService.getById(Integer.parseInt(payVo.getOutTradeNo()));
        if (Objects.isNull(member) || StrUtil.equals(Constants.IS_ACTIVE_FALSE, member.getIsActive())) {
            log.error("数据异常,会员不存在或者已过期" + payVo.getOutTradeNo());
            throw new BusinessException("数据缺失");
        }
        OrderPayInfo orderPayInfo = new OrderPayInfo()
                .setPayId(snowflake.nextId())
                .setMemberId(Integer.parseInt(payVo.getOutTradeNo()))
                .setMoney(payVo.getMoney())
                .setPaymentType(Constants.WECHAT_PAY)
                .setDescription(user.getUserName() + "-" + member.getMemberName())
                .setPayStatus(PayStatus.NOT.getValue())
                .setDeleteStatus(0)
                .setUserId(userId)
                .setCreateTime(LocalDateTime.now())
                .setUpdateTime(LocalDateTime.now());

        this.save(orderPayInfo);
        return orderPayInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean payNotice(Long payId, Boolean payResult) {
        log.info("支付回调信息===============> payId{} , payResult{}",payId,payResult);
        OrderPayInfo orderPayInfo = this.getById(payId);
        if (Objects.isNull(orderPayInfo)){
            throw new BusinessException("订单信息不存在");
        }
        orderPayInfo.setPayStatus(payResult?PayStatus.SUCCESS.getValue() : PayStatus.FAIL.getValue())
                .setUpdateTime(LocalDateTime.now());

        this.updateById(orderPayInfo);
        return Boolean.TRUE;
    }
}
