package com.jwl.driver.server.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.jwl.driver.server.dto.SecurityUser;
import com.jwl.driver.server.dto.TestQueryDto;
import com.jwl.driver.server.dto.TestSubmitDto;
import com.jwl.driver.server.entity.TdQuestionTest;
import com.jwl.driver.server.mapper.TdQuestionTestMapper;
import com.jwl.driver.server.service.ITdQuestionTestService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jwl.driver.server.util.SecurityUtil;
import com.jwl.driver.server.vo.TestTotalVo;
import io.swagger.models.auth.In;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.OptionalDouble;

/**
 * <p>
 * 模拟考试成绩表; 服务实现类
 * </p>
 *
 * @author Automated procedures
 * @since 2023-08-10
 */
@Service
public class TdQuestionTestServiceImpl extends ServiceImpl<TdQuestionTestMapper, TdQuestionTest> implements ITdQuestionTestService {

    @Override
    @Transactional(rollbackFor = Exception.class)
    public TdQuestionTest testSubmit(TestSubmitDto submitDto) {

        TdQuestionTest questionTest = new TdQuestionTest();
        BeanUtils.copyProperties(submitDto, questionTest);
        questionTest.setUserId(SecurityUtil.getUserId())
                .setCreateTime(LocalDateTime.now());

        return questionTest;
    }

    @Override
    public TestTotalVo testTotal(TestQueryDto queryDto) {
        TestTotalVo testTotalVo = new TestTotalVo();
        LambdaQueryWrapper<TdQuestionTest> queryWrapper = new LambdaQueryWrapper<TdQuestionTest>()
                .eq(TdQuestionTest::getCarTypeId, queryDto.getCarTypeId())
                .eq(TdQuestionTest::getSubject, queryDto.getSubject())
                .eq(TdQuestionTest::getUserId, SecurityUtil.getUserId())
                .orderByDesc(TdQuestionTest::getCreateTime);

        List<TdQuestionTest> list = this.list(queryWrapper);
        Long passNum = list.stream().filter(s -> s.getScore() >= 90).count();
        testTotalVo.setTestNum(list.size())
                .setPassNum(passNum.intValue());
        //取最近的五次
        if (CollectionUtil.isNotEmpty(list) && list.size() > 5) {
            list = list.subList(0, 5);
        }
        List<TdQuestionTest> recentTestList = CollectionUtil.reverse(list);
        //最近五次的平均分
        OptionalDouble averaged = recentTestList.stream().mapToInt(TdQuestionTest::getScore).average();
        Integer average = 0;
        if (averaged.isPresent()){
            Double averageScore = averaged.getAsDouble();
            average = averageScore.intValue();
        }
        testTotalVo.setRecentTestList(recentTestList)
                .setRecentAverageScore(average);
        return testTotalVo;
    }
}
